//-------------------------------------------------------------------
//	gl_texture - OpenGL Texture Class
//-------------------------------------------------------------------

#include "gl_texture.h"

#define NUMBER_MIPMAPS 3


gl_texture::gl_texture() {
}

gl_texture::~gl_texture() {

}

void gl_texture::use() {

	glBindTexture(GL_TEXTURE_2D, m_textureID);
}

bool gl_texture::load_TGA(char* file_name) {

	glGenTextures(1, &m_textureID);

	tga_image* image = new tga_image(file_name);
	if (image == NULL) {
		return false;
	}

	// figure out format
	int format;
	if (image->get_format() == LUMINANCE) {
		format = GL_LUMINANCE;
	}
	if (image->get_format() == RGBA) {
		format = GL_RGBA;
	}
	if (image->get_format() == RGB) {
		format = GL_RGB;
	}

	m_nWidth = image->get_width();
	m_nHeight = image->get_height();

	// get image data pointer
	unsigned char* data = image->get_ptr_image_data();

	glBindTexture (GL_TEXTURE_2D, m_textureID);
	glPixelStorei (GL_UNPACK_ALIGNMENT, 1);

	//TODO: experiment with different LOD bias for textures
	glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);

	gluBuild2DMipmaps(	GL_TEXTURE_2D,
						NUMBER_MIPMAPS,
						m_nWidth,
						m_nHeight,
						format,
						GL_UNSIGNED_BYTE,
						data);

	glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);

	delete(image); // data has been uploaded to video mem

	return true;
}

//
// creates a new texture from unsigned byte data
// format should be GL_RGBA, GL_RGB, GL_LUMINANCE, GL_ALPHA, etc...
//
bool gl_texture::create_raw_texture(unsigned char* data, GLint format, int width, int height) {

	glGenTextures(1, &m_textureID);

	glBindTexture (GL_TEXTURE_2D, m_textureID);
	glPixelStorei (GL_UNPACK_ALIGNMENT, 1);

	//TODO: experiment with different LOD bias for textures
	glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);

	gluBuild2DMipmaps(	GL_TEXTURE_2D,
						NUMBER_MIPMAPS,
						width,
						height,
						format,
						GL_UNSIGNED_BYTE,
						data);

	glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);

	return true;
}